#!/usr/bin/python
usage = '''Convert a moinmoin wiki to a trac wiki or multiple trac wikis
usage:

    moin2trac.py <wikiname> <moindir> <tracprojectdir> [<wikimapfile>]

    <wikiname> is the name of the moinmoin wiki to import.
    <moindir> is the original moinmoin directory.
    <tracprojectdir> is the target trac project directory
    <wikimapfile>  A mapping of page name to trac wikis
    
   Optionally you may also include <wikimapfile> which
   is useful if you would like to split your wiki into
   a trac multi-project or if you would prefer to map
   only some of you pages into your wiki.

   The syntax is as follow 
   MoinPageName   <tracprojectdir>

   If a <wikimapfile> is specified then only
   those pages in the map file will be transfered
'''

import sys,os
import re
import time
import socket
from trac.attachment import Attachment
from trac.util.text import to_unicode
# Work for 0.10.3 and 0.11
try:
    from  trac.scripts.admin import TracAdmin
except:
    from trac.admin.console import TracAdmin

sys.path.insert(0, '/etc/moin')
from MoinMoin.Page import Page
from MoinMoin.logfile import editlog
from MoinMoin import wikiutil
try:
    from MoinMoin.request.request_cli import Request
except:
    from MoinMoin.request import RequestCLI as Request

def convert(wikiname, moindir, tracdir = None, mapfile = None):
    pagemap = None
    if mapfile:
        pagemap = {}
        for line in open(mapfile):
            if line[0] == '#': continue
            (page, wikidir) = line.split()
            pagemap[page] = wikidir

    pages = os.listdir(moindir)
    for page in pages:
        page = wikiutil.unquoteWikiname(page)

        wikidir = tracdir
        if pagemap:
            if not pagemap.has_key(page): continue
            wikidir = pagemap[page]

        admin  = TracAdmin()
        admin.env_set (wikidir)
        moinrequest = Request(wikiname, str(page))
        moinpage = Page(moinrequest, page)

        if not moinpage.isStandardPage(): continue

        revisions = moinpage.getRevList()
        revisions.sort()
    
        logfile = editlog.EditLog(moinrequest, rootpagename=page)
        logfile.to_begin()

        for rev in revisions:
            revpage = Page(moinrequest, page, rev=rev)
            pagepath = revpage._text_filename()
            if os.access(pagepath, os.F_OK):
                cmd='wiki import %s %s' % ( page,  pagepath)
                print cmd, "->", wikidir

                name = 'trac'
                mtime = int(time.time())
                ip = '127.0.0.1'

                wanted_rev = "%08d" % rev
                for line in logfile:
                    if line.rev != wanted_rev: continue

                    # get edit info
                    kind, info = line.getInterwikiEditorData(moinrequest)
                    mtime = line.ed_time_usecs/1000000
                    name = line._usercache[line.userid].name
                    ip = line.addr
                    break

                f = open(pagepath,'r')
                data = to_unicode(f.read(), 'utf-8')
                admin.db_update("INSERT INTO wiki(version,name,time,author,ipnr,text) "
                   " SELECT 1+COALESCE(max(version),0),%s,%s,"
                   " %s,%s,%s FROM wiki "
                   " WHERE name=%s",
                   None, (page, mtime, name, ip, data, page))

        # Process attachments
        attdir = moinpage.getPagePath('attachments', check_create=0)
        if os.access(attdir, os.F_OK):
            attachments = os.listdir(attdir)
            for att in attachments:
                fname = wikiutil.url_quote(att)
                attachment = Attachment(admin.env_open(), 'wiki', page)

                stats = os.stat(attdir + '/'+ att)
                size = stats[6]

                name = 'trac'
                mtime = stats[8]
                ip = '127.0.0.1'

                logfile.to_end()
                for line in logfile.reverse():
                    if line.action == 'ATTNEW' and line.extra == fname:
                        kind, info = line.getInterwikiEditorData(moinrequest)
                        mtime = line.ed_time_usecs/1000000
                        name = line._usercache[line.userid].name
                        ip = line.addr
                        break

                print "attaching " + att + ' = ' + str(size)
                attachment.author = name
                attachment.ipnr = ip
                attfile = open (attdir + '/'+ att)
                attachment.insert (att, attfile, size, t=mtime)

if __name__ == '__main__':
    if len(sys.argv) < 3:
    	print usage
	sys.exit()
    args = sys.argv[1:]
    convert(*args)
